/**
 * @file
 * Shadow manager data types definition.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)            
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors: 
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef DATA_STRUCTURES__H
#define DATA_STRUCTURES__H

#include <Inventor/nodes/SoLight.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoEnvironment.h>


/**
 * User defined shadow computation method. 
 */
enum TShadowMethod {  
  zpass,  /**< Depth pass algorithm. */  
  zfail,  /**< Depth fail algorithm. */  
  autoset /**< TODO: Determine the shadow algorithm when running. */
};


/**
 * Algorithms used to compute and render shadows. 
 */
enum TShadowAlgorithm {
  zpassAlg, /**< Depth pass algorithm. */  
  zfailAlg  /**< Depth fail algorithm. */
};


/**
 * Shadow Manager settings. 
 */
struct TShadowSettings {  
  enum TShadowMethod method; /**< Algorithm used to create shadow volumes. */  
  bool dropShadows;          /**< Create shadow volumes or shadows. 
                                  Shadows are default. 
                                  For debug purposes only. */  
  float fDepth;              /**< Shadow volumes depth. */
};


/**
 * Defines all light types present in Open Inventor. 
 */
enum TLightTypes {
  TSoDirectionalLight, 
  TSoDirectionalLightManip,
  TSoPointLight,
  TSoPointLightManip,
  TSoSpotLight,
  TSoSpotLightManip,
  TUnknownLightType
};


/**
 * Light properties data structure. 
 */
struct TLightInfo {  
  SoLight * light;         /**< Pointer to the light. */  
  SbBool * status;         /**< Status. */
  SoEnvironment * globEnv; /**< Scene global environment node. */
};


/**
 * Light status and its switch node. 
 */
struct TLightSwitch {  
  SoSwitch * switchNode; /**< Switch node that enables/disables 
                              light shadow volumes traversal. */
  SbBool * status;       /**< Status. */
};


#endif