/**
 * @file
 * Shadow casting object model class definition.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)            
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors: 
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef OBJECT_MODEL__H
#define OBJECT_MODEL__H

//----------------------------------------------------------------------------
// INCLUDEs
//----------------------------------------------------------------------------

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoMatrixTransform.h>
//
#include <Inventor/actions/SoCallbackAction.h>
//
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/SbLinear.h>
//
#include <Inventor/elements/SoShapeHintsElement.h>
//
#include <Inventor/misc/SoChildList.h> 


//----------------------------------------------------------------------------
// IMPLEMENTATION
//----------------------------------------------------------------------------

/**
 * Scene object model consisting only of SoIndexedFaceSet. 
 * 
 * This model is created to be able generate shadow volumes independently of 
 * scene object plygon type. Model is created by addTriangleCallback called
 * for every triangle. Model must be updated when the object shape changes
 * or object's scale is updated.
 */
class CObjectModel
{
  private:
    /** Object model. */
    SoSeparator * objectModelRoot;
    /** Number of triangles of the object. */
    int32_t nTriangles; 

    static SbVec3f faceNormal(const SbVec3f p1, const SbVec3f p2, const SbVec3f p3);

    static void TriangleCB(void * userdata, SoCallbackAction * action,
                       const SoPrimitiveVertex * v1,
                       const SoPrimitiveVertex * v2,
                       const SoPrimitiveVertex * v3);

  public:
    /** Object model coordinates. */
    SoCoordinate3 * modelCoord3;
    /** Object model polygon indices. */
    SoIndexedFaceSet * modelIfs;
    /** Object model normals. */
    SoNormal * modelNormal;
    /** Object model polygon index. */
    int coord3idx;

    /** Previous normal. */
    static SbVec3f prevNormal;

    CObjectModel();
    ~CObjectModel();

    void createFaceModel(SoSeparator * object);
    SoSeparator * getFaceModelNode();
    int32_t getTrianglesCount();
};

#endif
