/**
 * @file
 * Shadow casting object class definition.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors:
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef SO_SHADOW_CASTER__H
#define SO_SHADOW_CASTER__H

//----------------------------------------------------------------------------
// INCLUDEs
//----------------------------------------------------------------------------

#include "ShadowManager.h"
#include "ObjectModel.h"
//
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotationXYZ.h>
//
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoWriteAction.h>
//
#include <Inventor/elements/SoShapeHintsElement.h>
//
#include <Inventor/misc/SoChildList.h>
//
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/SbLinear.h>

//----------------------------------------------------------------------------
// IMPLEMENTATION
//----------------------------------------------------------------------------

/**
 * Class defining a shadow caster node of the Shadow Manager.
 *
 * When a shadow casting scene object is added into the Shadow Manager,
 * a new node called Shadow Caster is created for it. 
 * It contains object's model and it is a parent node of 
 * all object's shadow volume nodes.
 */
class SoShadowCaster : public SoSeparator
{
  public:
    /** Object model. */
    CObjectModel * objectModel;
    /** Actual transformation matrix of the object. */
    SbMatrix trMatrix;
    /** Previous transformation matrix of the object. */
    SbMatrix trMatrixOld;
    /** Path from scene root to the object node. */
    SoPath * path;
    /** Shadow casting object root node. */
    SoSeparator * objectRoot;

  private:
    /** Pointer to shadow manager. */
    CShadowManager * manager;
    /** Object name index. */
    int nameIndex;
    /** Sensor of object movement. */
    SoPathSensor * moveSensor;
    /** Sensor of object change. */
    SoNodeSensor * shapeSensor;
    /** Object move flag. */
    bool hasMoved;

    bool getTransformationMatrix();

  public:    
    SoShadowCaster(CShadowManager * sm, SoSeparator * or);
    ~SoShadowCaster();

    SoSeparator * getObjectNode();
    CObjectModel * getObjectModel();
    SbMatrix * getMatrix();
    void setCasterName(int objectIndex);
    int getCasterName();

    void setMoveSensor(SoPathSensor * sensor);
    void setShapeSensor(SoNodeSensor * sensor);
    SoPathSensor * getMoveSensor();
    SoNodeSensor * getShapeSensor();

    void createFaceModel();
    void showFaceModel();
    void createShadowVolumes();
    
    void setMoved(bool m);
    bool moved();

};


#endif
