/**
 * @file
 * HUD display that shows text on the screen.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)            
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors: 
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef CHUD__H
#define CHUD__H

#include <Inventor/nodes/SoText2.h> 
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoFont.h> 


class CHUD
{	
  char * buf;
	SoText2 * text;
public:	
	void init(SoGroup *root );
	void refresh(char * data);	
};


void CHUD::init(SoGroup *root)
{
	SoSeparator * sep = new SoSeparator;
	root->addChild(sep);

	SoOrthographicCamera * camera = new SoOrthographicCamera;
	
	camera->position.setValue(SbVec3f(0.0f, 0.0f, 10.01f));
	camera->nearDistance = 10.0;
	camera->farDistance = 12.0;	
	sep->addChild(camera);

	// lighting model
	SoLightModel * lmodel = new SoLightModel;
	lmodel->model.setValue(SoLightModel::BASE_COLOR);
	sep->addChild(lmodel);

	// text node
	SoSeparator * txtsep = new SoSeparator;
	sep->addChild(txtsep);

	SoTranslation * move = new SoTranslation;
	move->translation.setValue(-0.3f, -0.97f, 0.0f);
	txtsep->addChild(move);

	SoFont * font = new SoFont;
	font->name.setValue("Arial");
	font->size.setValue(0.7f);
	txtsep->addChild(font);
	
	text = new SoText2;
  //text->justification.setValue(SoText2::LEFT);
	txtsep->addChild(text);
}

void CHUD::refresh(char * data)
{
  buf = data;
//	sprintf(buf,"%3i %3i", tank.skore, tank2.skore);
	text->string.setValue(buf);

/*
  text->spacing.setValue();
  text->justification.setValue();
*/

}

#endif