/**
 * @file
 * Scene creation and user callbacks header.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)            
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors: 
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */

#ifndef SCENE__H
#define SCENE__H

// application! default values 
#define SURFACE_SIZE2 100 // 75 
#define SURFACE_DEPTH -50
#define FAR_DEPTH 150.f

#include "ShadowEngine/ShadowManager.h"
//
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoPolygonOffset.h> 
#include <Inventor/nodes/SoScale.h> 
//
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoOutput.h>
//
#include <Inventor/manips/SoPointLightManip.h>
#include <Inventor/manips/SoSpotLightManip.h>
//
#include <Inventor/events/SoKeyboardEvent.h>
//
#include <Inventor/sensors/SoNodeSensor.h> 
//
#include <Inventor/Win/viewers/SoWinViewer.h> 
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
//
#include <Inventor/engines/SoTimeCounter.h> 
#include <Inventor/engines/SoElapsedTime.h> 
#include <Inventor/engines/SoCompose.h>

/**
 * Surface coordinates.
 */
static float surfaceVertices[8][3] =
{
  -SURFACE_SIZE2, -SURFACE_SIZE2, 0,  // bottom
   SURFACE_SIZE2, -SURFACE_SIZE2, 0,
   SURFACE_SIZE2,  SURFACE_SIZE2, 0,  // back
  -SURFACE_SIZE2,  SURFACE_SIZE2, 0,
  -SURFACE_SIZE2,  SURFACE_SIZE2, 2*SURFACE_SIZE2,  // right
   SURFACE_SIZE2,  SURFACE_SIZE2, 2*SURFACE_SIZE2,
  -SURFACE_SIZE2, -SURFACE_SIZE2, 2*SURFACE_SIZE2,  // left
   SURFACE_SIZE2, -SURFACE_SIZE2, 2*SURFACE_SIZE2,
};


/**
 * Surface indices.
 */
static int32_t surfaceIndices[8][4] =
{
  0, 1, 2, -1, // bottom
  3, 0, 2, -1,
  2, 5, 4, -1, // back
  3, 2, 4, -1,
  3, 4, 6, -1, // left
  0, 3, 6, -1,
  2, 1, 7, -1, // right
  5, 2, 7, -1,
};


// Compute normal vector from 3 points, counterclockwise
// +++ do stinu
// SbVec3f faceNormal(const SbVec3f p1, const SbVec3f p2, const SbVec3f p3);

// Scene node
SoSeparator* createScene(int argc, char **argv, SoWinViewer *viewer);

// Surface node
SoSeparator* createSurface(float depth, SbColor color);


#endif
