/**
 * @file
 * Shadows demonstration application.
 *
 * @author Tom "Shamot" Burian
 */
/* ---------------------------------------------------------------------------
 * Authors:      Tom Burian (tburian _AT centrum.cz)            
 * Thanks to:    PCJohn (peciva _AT fit.vutbr.cz)
 * Contributors: 
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 * ---------------------------------------------------------------------------
 */


//----------------------------------------------------------------------------
// INCLUDEs
//----------------------------------------------------------------------------

#include <Inventor/Win/SoWin.h>
#include <Inventor/Win/viewers/SoWinExaminerViewer.h>
#include <Inventor/SoDB.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/SoOutput.h>
//
#include "shadows_demo.h"
#include "scene.h"


//----------------------------------------------------------------------------
// IMPLEMENTATION
//----------------------------------------------------------------------------

// Linker: Link with main (not WinMain)
// and use a console
#pragma comment(linker, "/entry:\"mainCRTStartup\"")
#pragma comment(linker, "/subsystem:\"console\"")

/** Scene creation. */
extern SoSeparator* createScene(int argc, char **argv);
char title[50] = "";
const char *sceneFilename = "scene_graph.iv"; /**< Scene graph iv output file. */

/**
 * Main application.
 */
int main(int argc, char **argv)
{
  SoDB::init();

  // Show FPS
  //putenv("IV_SEPARATOR_MAX_CACHES=0");
  putenv("COIN_SHOW_FPS_COUNTER=1");

  // Initialisation
  HWND window = SoWin::init(argv[0]);
  if (window == NULL) exit(1);

  // Root of the scene
  SoSeparator *root = createScene(argc, argv);
  root->ref();

  ONDEBUG(
    // Debug: write out scene graph into a file "scene_graph.iv"
    SoOutput output;
    output.openFile(sceneFilename);
    SoWriteAction * writer = new SoWriteAction(&output);
    writer->apply(root);
    delete writer;
  );

  bool headlight = FALSE;
  bool feedbackVisibility = TRUE;

  // Create viewer
  SoWinExaminerViewer *viewer = new SoWinExaminerViewer(window);
  viewer->setHeadlight(headlight);
  viewer->setFeedbackVisibility(feedbackVisibility);
  viewer->setBackgroundColor( VIEWER_BGCOLOR );

  // IMPORTANT: user must enable stencil buffer to see shadows!!!
  viewer->setStencilBuffer(TRUE); 

  // set up viewer
  viewer->setSceneGraph(root);
  sprintf(title,"%s %s", APPNAME, VERSION);
  viewer->setTitle(title);

  viewer->show();
 
  // Show window and start render loop
  SoWin::show(window);
  SoWin::mainLoop();

  // Free memory
  delete viewer;
  root->unref();

  return 0;
}

//////////////////////////////////////////////////////////////////////////////
